@@ -114,26 +114,22 @@ def parse_uri(config_uri: str) -> Optional[ConfigParams]:
             "port": 0,
         }
 
-        if "@" not in main_part:
-            if protocol == "vmess":
-                pass
-            else:
-                match_addr = re.search(r"://([^:]+):(\d+)", main_part)
-                if not match_addr:
-                    logging.warning(
-                        f"Invalid URI structure (missing @). Skipping: {uri[:50]}..."
-                    )
-                    return None
-
-        match = re.search(r"@([^:]+):(\d+)", main_part.split("?")[0])
-        if match:
-            common["address"] = match.group(1)
-            common["port"] = int(match.group(2))
+        at_match = re.search(r"@([^:]+):(\d+)", main_part)
+        if at_match:
+            common["address"] = at_match.group(1)
+            common["port"] = int(at_match.group(2))
 
         params = parser(uri, common)
-        if params and not params.display_tag:
-            params.display_tag = decoded_display_tag
-        if protocol == "mvless" and params:
+
+        if not params:
+            return None
+        if not params.address or not params.port or params.port <= 0:
+            logging.warning(
+                f"Parsed config '{params.display_tag}' is missing a valid address or port. Skipping."
+            )
+            return None
+
+        if protocol == "mvless":
             _parse_mvless_extensions(params, uri)
         return params
     except Exception as e:
@@ -150,11 +146,19 @@ def _parse_vless(uri: str, common: dict) -> Optional[ConfigParams]:
 
     network_type = params.get("type", "tcp")
     path = params.get("path", "")
+    host = params.get("host", "")
+    sni = params.get("sni", host)
+
     if network_type == "grpc" and (not path or path == "/"):
-        logging.warning(
-            f"gRPC config '{common['display_tag']}' is missing a valid serviceName (path). Skipping."
+        path = sni or host
+        if not path or path == "/":
+            logging.warning(
+                f"gRPC config '{common['display_tag']}' is missing a valid serviceName (path) and has no fallback SNI/Host. Skipping."
+            )
+            return None
+        logging.info(
+            f"gRPC config '{common['display_tag']}' missing serviceName. Using fallback: '{path}'"
         )
-        return None
 
     return ConfigParams(
         **common,
@@ -170,8 +174,6 @@ def _parse_vless(uri: str, common: dict) -> Optional[ConfigParams]:
         flow=params.get("flow", ""),
         encryption=params.get("encryption", "none"),
     )
-
-
 def _parse_mvless_extensions(params: ConfigParams, uri: str):
     """Parses Mux and Fragment parameters specific to the Mvless protocol and modifies the ConfigParams object."""
     try:
@@ -203,12 +205,20 @@ def _parse_vmess(uri: str, common: dict) -> Optional[ConfigParams]:
 
         network_type = decoded.get("net", "tcp")
         path = decoded.get("path", "")
+        host = decoded.get("host", "")
+        sni = decoded.get("sni", "")
         display_tag = decoded.get("ps", common["display_tag"])
+
         if network_type == "grpc" and (not path or path == "/"):
-            logging.warning(
-                f"gRPC config '{display_tag}' is missing a valid serviceName (path). Skipping."
+            path = sni or host
+            if not path or path == "/":
+                logging.warning(
+                    f"gRPC config '{display_tag}' is missing a valid serviceName (path) and has no fallback SNI/Host. Skipping."
+                )
+                return None
+            logging.info(
+                f"gRPC config '{display_tag}' missing serviceName. Using fallback: '{path}'"
             )
-            return None
 
         vmess_internal_safe_tag = common["tag"]
         if "ps" in decoded:
@@ -236,23 +246,38 @@ def _parse_vmess(uri: str, common: dict) -> Optional[ConfigParams]:
         logging.error(f"Failed to parse VMess URI '{uri[:30]}...': {e}. Skipping.")
         return None
 
-
 def _parse_trojan(uri: str, common: dict) -> ConfigParams:
     parsed_url = urllib.parse.urlparse(uri)
     params = _parse_query_params(parsed_url.query)
+
+    network_type = params.get("type", "tcp")
+    path = params.get("path", "/")
+    host = params.get("host", "")
+    sni = params.get("sni", common.get("address", ""))
+
+    if network_type == "grpc" and (not path or path == "/"):
+        path = sni or host
+        if not path or path == "/":
+            logging.warning(
+                f"gRPC config '{common['display_tag']}' is missing a valid serviceName (path) and has no fallback SNI/Host. Skipping."
+            )
+            return None
+        logging.info(
+            f"gRPC config '{common['display_tag']}' missing serviceName. Using fallback: '{path}'"
+        )
+
     return ConfigParams(
         **common,
         password=parsed_url.username,
         sni=params.get("sni", common["address"]),
-        network=params.get("type", "tcp"),
+        network=network_type,
         security=params.get("security", "tls"),
         fp=params.get("fp", ""),
         header_type=params.get("headerType", "none"),
-        host=params.get("host", ""),
-        path=params.get("path", "/"),
+        host=host,
+        path=path,
     )
 
-
 def _parse_shadowsocks(uri: str, common: dict) -> Optional[ConfigParams]:
     try:
         main_part = uri.split("#")[0].replace("ss://", "")
